@@ -9,9 +9,10 @@ void mp_msg_uninit(struct mpv_global *global);
 void mp_msg_update_msglevels(struct mpv_global *global);
 void mp_msg_mute(struct mpv_global *global, bool mute);
 void mp_msg_force_stderr(struct mpv_global *global, bool force_stderr);
-void mp_msg_flush_status_line(struct mpv_global *global);
 bool mp_msg_has_status_line(struct mpv_global *global);
 
+void mp_msg_flush_status_line(struct mp_log *log);
+
 struct mp_log_buffer_entry {
     char *prefix;
     int level;
@@ -32,5 +33,6 @@ struct bstr;
 int mp_msg_split_msglevel(struct bstr *s, struct bstr *out_mod, int *out_level);
 
 extern const char *const mp_log_levels[MSGL_MAX + 1];
+extern const int mp_mpv_log_levels[MSGL_MAX + 1];
 
 #endif
