@@ -18,6 +18,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <strings.h>
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -32,11 +33,12 @@
 #include "stream/stream.h"
 #include "demux.h"
 #include "stheader.h"
+#include "codec_tags.h"
 #include "mf.h"
 
 #define MF_MAX_FILE_SIZE (1024 * 1024 * 256)
 
-static void demux_seek_mf(demuxer_t *demuxer, float rel_seek_secs, int flags)
+static void demux_seek_mf(demuxer_t *demuxer, double rel_seek_secs, int flags)
 {
     mf_t *mf = demuxer->priv;
     int newpos = (flags & SEEK_ABSOLUTE) ? 0 : mf->curr_frame - 1;
@@ -48,7 +50,7 @@ static void demux_seek_mf(demuxer_t *demuxer, float rel_seek_secs, int flags)
     if (newpos < 0)
         newpos = 0;
     if (newpos >= mf->nr_of_files)
-        newpos = mf->nr_of_files - 1;
+        newpos = mf->nr_of_files;
     mf->curr_frame = newpos;
 }
 
@@ -76,10 +78,12 @@ static int demux_mf_fill_buffer(demuxer_t *demuxer)
         bstr data = stream_read_complete(stream, NULL, MF_MAX_FILE_SIZE);
         if (data.len) {
             demux_packet_t *dp = new_demux_packet(data.len);
-            memcpy(dp->buffer, data.start, data.len);
-            dp->pts = mf->curr_frame / mf->sh->fps;
-            dp->keyframe = true;
-            demuxer_add_packet(demuxer, demuxer->streams[0], dp);
+            if (dp) {
+                memcpy(dp->buffer, data.start, data.len);
+                dp->pts = mf->curr_frame / mf->sh->fps;
+                dp->keyframe = true;
+                demux_add_packet(demuxer->streams[0], dp);
+            }
         }
         talloc_free(data.start);
     }
@@ -183,7 +187,10 @@ static int demux_open_mf(demuxer_t *demuxer, enum demux_check check)
     if (!mf || mf->nr_of_files < 1)
         goto error;
 
-    const char *codec = probe_format(mf, demuxer->opts->mf_type, check);
+    char *force_type = demuxer->opts->mf_type;
+    const char *codec = mp_map_mimetype_to_video_codec(demuxer->stream->mime_type);
+    if (!codec || (force_type && force_type[0]))
+        codec = probe_format(mf, force_type, check);
     if (!codec)
         goto error;
 
