@@ -55,6 +55,11 @@ typedef struct m_config {
     struct m_config_option *opts; // all options, even suboptions
     int num_opts;
 
+    // Creation parameters
+    size_t size;
+    const void *defaults;
+    const struct m_option *options;
+
     // List of defined profiles.
     struct m_profile *profiles;
     // Depth when recursively including profiles.
@@ -87,6 +92,9 @@ struct m_config *m_config_new(void *talloc_ctx, struct mp_log *log,
                               size_t size, const void *defaults,
                               const struct m_option *options);
 
+// (Warning: new object references config->log and others.)
+struct m_config *m_config_dup(void *talloc_ctx, struct m_config *config);
+
 struct m_config *m_config_from_obj_desc(void *talloc_ctx, struct mp_log *log,
                                         struct m_obj_desc *desc);
 
