@@ -22,7 +22,27 @@ function mp.get_opt(key, def)
     return val
 end
 
-local callbacks = {}
+-- For dispatching script_binding. This is sent as:
+--      script_message_to $script_name $binding_name $keystate
+-- The array is indexed by $binding_name, and has functions like this as value:
+--      fn($binding_name, $keystate)
+local dispatch_key_bindings = {}
+
+local message_id = 0
+local function reserve_binding()
+    message_id = message_id + 1
+    return "__keybinding" .. tostring(message_id)
+end
+
+local function dispatch_key_binding(name, state)
+    local fn = dispatch_key_bindings[name]
+    if fn then
+        fn(name, state)
+    end
+end
+
+-- "Old", deprecated API
+
 -- each script has its own section, so that they don't conflict
 local default_section = "input_dispatch_" .. mp.script_name
 
@@ -34,13 +54,8 @@ local default_section = "input_dispatch_" .. mp.script_name
 -- Note: the bindings are not active by default. Use enable_key_bindings().
 --
 -- list is an array of key bindings, where each entry is an array as follow:
---      {key, callback}
---      {key, callback, callback_down}
+--      {key, callback_press, callback_down, callback_up}
 -- key is the key string as used in input.conf, like "ctrl+a"
--- callback is a Lua function that is called when the key binding is used.
--- callback_down can be given too, and is called when a mouse button is pressed
--- if the key is a mouse button. (The normal callback will be for mouse button
--- down.)
 --
 -- callback can be a string too, in which case the following will be added like
 -- an input.conf line: key .. " " .. callback
@@ -52,10 +67,28 @@ function mp.set_key_bindings(list, section, flags)
         local key = entry[1]
         local cb = entry[2]
         local cb_down = entry[3]
-        if type(cb) == "function" then
-            callbacks[#callbacks + 1] = {press=cb, before_press=cb_down}
-            cfg = cfg .. key .. " script_dispatch " .. mp.script_name
-                  .. " " .. #callbacks .. "\n"
+        local cb_up = entry[4]
+        if type(cb) ~= "string" then
+            local mangle = reserve_binding()
+            dispatch_key_bindings[mangle] = function(name, state)
+                local event = state:sub(1, 1)
+                local is_mouse = state:sub(2, 2) == "m"
+                local def = (is_mouse and "u") or "d"
+                if event == "r" then
+                    return
+                end
+                if event == "p" and cb then
+                    cb()
+                elseif event == "d" and cb_down then
+                    cb_down()
+                elseif event == "u" and cb_up then
+                    cb_up()
+                elseif event == def and cb then
+                    cb()
+                end
+            end
+            cfg = cfg .. key .. " script_binding " ..
+                  mp.script_name .. "/" .. mangle .. "\n"
         else
             cfg = cfg .. key .. " " .. cb .. "\n"
         end
@@ -75,21 +108,9 @@ function mp.set_mouse_area(x0, y0, x1, y1, section)
     mp.input_set_section_mouse_area(section or default_section, x0, y0, x1, y1)
 end
 
-local function script_dispatch(event)
-    local cb = callbacks[event.arg0]
-    if cb then
-        if event.type == "press" and cb.press then
-            cb.press()
-        elseif event.type == "keyup_follows" and cb.before_press then
-            cb.before_press()
-        end
-    end
-end
-
 -- "Newer" and more convenient API
 
 local key_bindings = {}
-local message_id = 1
 
 local function update_key_bindings()
     for i = 1, 2 do
@@ -105,29 +126,71 @@ local function update_key_bindings()
         local cfg = ""
         for k, v in pairs(key_bindings) do
             if v.forced ~= def then
-                cfg = cfg .. v.key .. " script_message_to " .. mp.script_name
-                      .. " " .. v.name .. "\n"
+                cfg = cfg .. v.bind .. "\n"
             end
         end
         mp.input_define_section(section, cfg, flags)
         -- TODO: remove the section if the script is stopped
-        mp.input_enable_section(section)
+        mp.input_enable_section(section, "allow-hide-cursor|allow-vo-dragging")
     end
 end
 
-local function add_binding(attrs, key, name, fn)
+local function add_binding(attrs, key, name, fn, rp)
+    rp = rp or ""
     if (type(name) ~= "string") and (not fn) then
         fn = name
-        name = "message" .. tostring(message_id)
-        message_id = message_id + 1
+        name = reserve_binding()
     end
-    attrs.key = key
+    local bind = key
+    local repeatable = rp == "repeatable" or rp["repeatable"]
+    if rp["forced"] then
+        attrs.forced = true
+    end
+    local key_cb, msg_cb
+    if not fn then
+        fn = function() end
+    end
+    if rp["complex"] then
+        local key_states = {
+            ["u"] = "up",
+            ["d"] = "down",
+            ["r"] = "repeat",
+            ["p"] = "press",
+        }
+        key_cb = function(name, state)
+            fn({
+                event = key_states[state:sub(1, 1)] or "unknown",
+                is_mouse = state:sub(2, 2) == "m"
+            })
+        end
+        msg_cb = function()
+            fn({event = "press", is_mouse = false})
+        end
+    else
+        key_cb = function(name, state)
+            -- Emulate the same semantics as input.c uses for most bindings:
+            -- For keyboard, "down" runs the command, "up" does nothing;
+            -- for mouse, "down" does nothing, "up" runs the command.
+            -- Also, key repeat triggers the binding again.
+            local event = state:sub(1, 1)
+            local is_mouse = state:sub(2, 2) == "m"
+            if event == "r" and not repeatable then
+                return
+            end
+            if is_mouse and (event == "u" or event == "p") then
+                fn()
+            elseif (not is_mouse) and (event == "d" or event == "r" or event == "p") then
+                fn()
+            end
+        end
+        msg_cb = fn
+    end
+    attrs.bind = bind .. " script_binding " .. mp.script_name .. "/" .. name
     attrs.name = name
     key_bindings[name] = attrs
     update_key_bindings()
-    if fn then
-        mp.register_script_message(name, fn)
-    end
+    dispatch_key_bindings[name] = key_cb
+    mp.register_script_message(name, msg_cb)
 end
 
 function mp.add_key_binding(...)
@@ -320,10 +383,12 @@ end
 
 -- default handlers
 mp.register_event("shutdown", function() mp.keep_running = false end)
-mp.register_event("script-input-dispatch", script_dispatch)
 mp.register_event("client-message", message_dispatch)
 mp.register_event("property-change", property_change)
 
+-- sent by "script_binding"
+mp.register_script_message("key-binding", dispatch_key_binding)
+
 mp.msg = {
     log = mp.log,
     fatal = function(...) return mp.log("fatal", ...) end,
@@ -399,7 +464,30 @@ function mp.osd_message(text, duration)
     mp.commandv("show_text", text, duration)
 end
 
-function mp.format_table(t, set)
+local hook_table = {}
+local hook_registered = false
+
+local function hook_run(id, cont)
+    local fn = hook_table[tonumber(id)]
+    if fn then
+        fn()
+    end
+    mp.commandv("hook_ack", cont)
+end
+
+function mp.add_hook(name, pri, cb)
+    if not hook_registered then
+        mp.register_script_message("hook_run", hook_run)
+        hook_registered = true
+    end
+    local id = #hook_table + 1
+    hook_table[id] = cb
+    mp.commandv("hook_add", name, id, pri)
+end
+
+local mp_utils = package.loaded["mp.utils"]
+
+function mp_utils.format_table(t, set)
     if not set then
         set = { [t] = true }
     end
@@ -422,30 +510,33 @@ function mp.format_table(t, set)
             vals[#keys] = v
         end
     end
-    local function fmtval(v)
-        if type(v) == "string" then
-            return "\"" .. v .. "\""
-        elseif type(v) == "table" then
-            if set[v] then
-                return "[cycle]"
-            end
-            set[v] = true
-            return mp.format_table(v, set)
-        else
-            return tostring(v)
-        end
-    end
     for i = 1, #keys do
         if #res > 1 then
             res = res .. ", "
         end
         if i > arr then
-            res = res .. fmtval(keys[i]) .. " = "
+            res = res .. mp_utils.to_string(keys[i], set) .. " = "
         end
-        res = res .. fmtval(vals[i])
+        res = res .. mp_utils.to_string(vals[i], set)
     end
     res = res .. "}"
     return res
 end
 
+function mp_utils.to_string(v, set)
+    if type(v) == "string" then
+        return "\"" .. v .. "\""
+    elseif type(v) == "table" then
+        if set then
+            if set[v] then
+                return "[cycle]"
+            end
+            set[v] = true
+        end
+        return mp_utils.format_table(v, set)
+    else
+        return tostring(v)
+    end
+end
+
 return {}
