@@ -21,6 +21,8 @@
 #include <inttypes.h>
 #include <assert.h>
 #include <dirent.h>
+#include <string.h>
+#include <strings.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
@@ -59,7 +61,20 @@ static int cmp_entry(const void *pa, const void *pb)
     return 0;
 }
 
-static char **find_files(const char *original_file, const char *suffix)
+static bool test_matroska_ext(const char *filename)
+{
+    static const char *const exts[] = {".mkv", ".mka", ".mks", ".mk3d", NULL};
+    for (int n = 0; exts[n]; n++) {
+        const char *suffix = exts[n];
+        int offset = strlen(filename) - strlen(suffix);
+        // name must end with suffix
+        if (offset > 0 && strcasecmp(filename + offset, suffix) == 0)
+            return true;
+    }
+    return false;
+}
+
+static char **find_files(const char *original_file)
 {
     void *tmpmem = talloc_new(NULL);
     char *basename = mp_basename(original_file);
@@ -75,9 +90,7 @@ static char **find_files(const char *original_file, const char *suffix)
     struct dirent *ep;
     int num_results = 0;
     while ((ep = readdir(dp))) {
-        int suffix_offset = strlen(ep->d_name) - strlen(suffix);
-        // name must end with suffix
-        if (suffix_offset < 0 || strcmp(ep->d_name + suffix_offset, suffix))
+        if (!test_matroska_ext(ep->d_name))
             continue;
         // don't list the original name
         if (!strcmp(ep->d_name, basename))
@@ -269,7 +282,7 @@ static int find_ordered_chapter_sources(struct MPContext *mpctx,
         } else {
             MP_INFO(mpctx, "Will scan other files in the "
                    "same directory to find referenced sources.\n");
-            filenames = find_files(main_filename, ".mkv");
+            filenames = find_files(main_filename);
             num_filenames = MP_TALLOC_ELEMS(filenames);
             talloc_steal(tmp, filenames);
         }
@@ -367,7 +380,8 @@ static void build_timeline_loop(struct MPContext *mpctx,
                                 uint64_t *missing_time,
                                 uint64_t *last_end_time,
                                 struct timeline_part **timeline,
-                                struct chapter *chapters,
+                                struct demux_chapter *chapters,
+                                int num_chapters,
                                 int *part_count,
                                 uint64_t skip,
                                 uint64_t limit)
@@ -405,8 +419,12 @@ static void build_timeline_loop(struct MPContext *mpctx,
              * recursive ordered chapter editions? If so, more code will be
              * needed to add chapters for external non-ordered segment loading
              * as well since that part is not recursive. */
+
+            if (i >= num_chapters)
+                break; // probably needed only for broken sources
+
             if (!limit) {
-                chapters[i].start = *starttime / 1e9;
+                chapters[i].pts = *starttime / 1e9;
                 chapters[i].name = talloc_strdup(chapters, c->name);
             }
 
@@ -464,11 +482,9 @@ static void build_timeline_loop(struct MPContext *mpctx,
             } else {
                 build_timeline_loop(mpctx, sources, num_sources, j, starttime,
                                     missing_time, last_end_time, timeline,
-                                    chapters, part_count, c->start, c->end);
-                /* The loop call has added time as needed (we can't add it here
-                 * due to 'join_diff' in the add_timeline_part function. Since
-                 * the time has already been added as needed, the chapter has
-                 * an effective 0 length at this point. */
+                                    chapters, num_chapters, part_count,
+                                    c->start, c->end);
+                // Already handled by the loop call.
                 chapter_length = 0;
             }
             *last_end_time = c->end;
@@ -547,22 +563,33 @@ void build_ordered_chapter_timeline(struct MPContext *mpctx)
     talloc_free(uids);
 
     struct timeline_part *timeline = talloc_array_ptrtype(NULL, timeline, 0);
-    struct chapter *chapters =
-        talloc_zero_array(NULL, struct chapter, m->num_ordered_chapters);
+    struct demux_chapter *chapters =
+        talloc_zero_array(NULL, struct demux_chapter, m->num_ordered_chapters);
+    // Stupid hack, because fuck everything.
+    for (int n = 0; n < m->num_ordered_chapters; n++)
+        chapters[n].pts = -1;
     uint64_t starttime = 0;
     uint64_t missing_time = 0;
     uint64_t last_end_time = 0;
     int part_count = 0;
     build_timeline_loop(mpctx, sources, num_sources, 0, &starttime,
                         &missing_time, &last_end_time, &timeline,
-                        chapters, &part_count, 0, 0);
+                        chapters, m->num_ordered_chapters, &part_count, 0, 0);
+
+    // Fuck everything (2): filter out all "unset" chapters.
+    for (int n = m->num_ordered_chapters - 1; n >= 0; n--) {
+        if (chapters[n].pts == -1)
+            MP_TARRAY_REMOVE_AT(chapters, m->num_ordered_chapters, n);
+    }
 
     if (!part_count) {
-        // None of the parts come from the file itself???
-        talloc_free(sources);
-        talloc_free(timeline);
-        talloc_free(chapters);
-        return;
+        // None of  the parts come from the file itself???
+        // Broken file, but we need at least 1 valid timeline part - add a dummy.
+        MP_WARN(mpctx, "Ordered chapters file with no parts?\n");
+        struct timeline_part new = {
+            .source = demuxer,
+        };
+        MP_TARRAY_APPEND(NULL, timeline, part_count, new);
     }
 
     struct timeline_part new = {
