@@ -241,7 +241,7 @@ static bool cache_fill(struct priv *s)
     int64_t space = s->buffer_size - (newb + back);
 
     // offset into the buffer that maps to max_filepos
-    int pos = s->max_filepos - s->offset;
+    int64_t pos = s->max_filepos - s->offset;
     if (pos >= s->buffer_size)
         pos -= s->buffer_size; // wrap-around
 
@@ -651,8 +651,7 @@ int stream_cache_init(stream_t *cache, stream_t *stream,
     s->seekable = stream->seekable;
 
     if (pthread_create(&s->cache_thread, NULL, cache_thread, s) != 0) {
-        MP_ERR(s, "Starting cache process/thread failed: %s.\n",
-               strerror(errno));
+        MP_ERR(s, "Starting cache thread failed.\n");
         return -1;
     }
     s->cache_thread_running = true;
