@@ -20,7 +20,6 @@
 #include <libavformat/avio.h>
 #include <libavutil/opt.h>
 
-#include "config.h"
 #include "options/options.h"
 #include "options/path.h"
 #include "common/msg.h"
@@ -34,8 +33,6 @@
 #include "misc/bstr.h"
 #include "talloc.h"
 
-struct stream_lavf_params *stream_lavf_opts;
-
 #define OPT_BASE_STRUCT struct stream_lavf_params
 struct stream_lavf_params {
     char **avopts;
@@ -49,6 +46,8 @@ const struct m_sub_options stream_lavf_conf = {
     .size = sizeof(struct stream_lavf_params),
 };
 
+static const char *const http_like[];
+
 static int open_f(stream_t *stream);
 static struct mp_tags *read_icy(stream_t *stream);
 
@@ -113,10 +112,16 @@ static int control(stream_t *s, int cmd, void *arg)
     case STREAM_CTRL_AVSEEK: {
         struct stream_avseek *c = arg;
         int64_t r = avio_seek_time(avio, c->stream_index, c->timestamp, c->flags);
-        if (r >= 0)
+        if (r >= 0) {
+            stream_drop_buffers(s);
             return 1;
+        }
         break;
     }
+    case STREAM_CTRL_HAS_AVSEEK:
+        if (avio->read_seek)
+            return 1;
+        break;
     case STREAM_CTRL_GET_METADATA: {
         *(struct mp_tags **)arg = read_icy(s);
         if (!*(struct mp_tags **)arg)
@@ -143,6 +148,60 @@ static int interrupt_cb(void *ctx)
 
 static const char * const prefix[] = { "lavf://", "ffmpeg://" };
 
+void mp_setup_av_network_options(AVDictionary **dict, struct mpv_global *global,
+                                 struct mp_log *log, struct MPOpts *opts)
+{
+    void *temp = talloc_new(NULL);
+
+    // HTTP specific options (other protocols ignore them)
+    if (opts->network_useragent)
+        av_dict_set(dict, "user-agent", opts->network_useragent, 0);
+    if (opts->network_cookies_enabled) {
+        char *file = opts->network_cookies_file;
+        if (file && file[0])
+            file = mp_get_user_path(temp, global, file);
+        char *cookies = cookies_lavf(temp, log, file);
+        if (cookies && cookies[0])
+            av_dict_set(dict, "cookies", cookies, 0);
+    }
+    av_dict_set(dict, "tls_verify", opts->network_tls_verify ? "1" : "0", 0);
+    if (opts->network_tls_ca_file)
+        av_dict_set(dict, "ca_file", opts->network_tls_ca_file, 0);
+    char *cust_headers = talloc_strdup(temp, "");
+    if (opts->network_referrer) {
+        cust_headers = talloc_asprintf_append(cust_headers, "Referer: %s\r\n",
+                                              opts->network_referrer);
+    }
+    if (opts->network_http_header_fields) {
+        for (int n = 0; opts->network_http_header_fields[n]; n++) {
+            cust_headers = talloc_asprintf_append(cust_headers, "%s\r\n",
+                                                  opts->network_http_header_fields[n]);
+        }
+    }
+    if (strlen(cust_headers))
+        av_dict_set(dict, "headers", cust_headers, 0);
+    av_dict_set(dict, "icy", "1", 0);
+
+    mp_set_avdict(dict, opts->stream_lavf_opts->avopts);
+
+    talloc_free(temp);
+}
+
+// Escape http URLs with unescaped, invalid characters in them.
+// libavformat's http protocol does not do this, and a patch to add this
+// in a 100% safe case (spaces only) was rejected.
+static char *normalize_url(void *ta_parent, const char *filename)
+{
+    bstr proto = mp_split_proto(bstr0(filename), NULL);
+    for (int n = 0; http_like[n]; n++) {
+        if (bstr_equals0(proto, http_like[n]))
+            // Escape everything but reserved characters.
+            // Also don't double-scape, so include '%'.
+            return mp_url_escape(ta_parent, filename, ":/?#[]@!$&'()*+,;=%");
+    }
+    return (char *)filename;
+}
+
 static int open_f(stream_t *stream)
 {
     struct MPOpts *opts = stream->opts;
@@ -186,41 +245,15 @@ static int open_f(stream_t *stream)
         filename = talloc_asprintf(temp, "mmsh://%.*s", BSTR_P(b_filename));
     }
 
-    // HTTP specific options (other protocols ignore them)
-    if (opts->network_useragent)
-        av_dict_set(&dict, "user-agent", opts->network_useragent, 0);
-    if (opts->network_cookies_enabled) {
-        char *file = opts->network_cookies_file;
-        if (file && file[0])
-            file = mp_get_user_path(temp, stream->global, file);
-        char *cookies = cookies_lavf(temp, stream->log, file);
-        if (cookies && cookies[0])
-            av_dict_set(&dict, "cookies", cookies, 0);
-    }
-    av_dict_set(&dict, "tls_verify", opts->network_tls_verify ? "1" : "0", 0);
-    if (opts->network_tls_ca_file)
-        av_dict_set(&dict, "ca_file", opts->network_tls_ca_file, 0);
-    char *cust_headers = talloc_strdup(temp, "");
-    if (opts->network_referrer) {
-        cust_headers = talloc_asprintf_append(cust_headers, "Referer: %s\r\n",
-                                              opts->network_referrer);
-    }
-    if (opts->network_http_header_fields) {
-        for (int n = 0; opts->network_http_header_fields[n]; n++) {
-            cust_headers = talloc_asprintf_append(cust_headers, "%s\r\n",
-                                                  opts->network_http_header_fields[n]);
-        }
-    }
-    if (strlen(cust_headers))
-        av_dict_set(&dict, "headers", cust_headers, 0);
-    av_dict_set(&dict, "icy", "1", 0);
-    mp_set_avdict(&dict, opts->stream_lavf_opts->avopts);
+    mp_setup_av_network_options(&dict, stream->global, stream->log, opts);
 
     AVIOInterruptCB cb = {
         .callback = interrupt_cb,
         .opaque = stream,
     };
 
+    filename = normalize_url(stream, filename);
+
     int err = avio_open2(&avio, filename, flags, &cb, &dict);
     if (err < 0) {
         if (err == AVERROR_PROTOCOL_NOT_FOUND)
@@ -318,6 +351,9 @@ done:
     return res;
 }
 
+static const char *const http_like[] =
+    {"http", "https", "mmsh", "mmshttp", "httproxy", NULL};
+
 const stream_info_t stream_info_ffmpeg = {
   .name = "ffmpeg",
   .open = open_f,
@@ -339,6 +375,7 @@ const stream_info_t stream_info_ffmpeg_unsafe = {
   .open = open_f,
   .protocols = (const char *const[]){
      "lavf", "ffmpeg", "udp", "ftp", "tcp", "tls", "unix", "sftp", "md5",
+     "concat",
      NULL },
   .can_write = true,
 };
