@@ -16,7 +16,7 @@
 
 static const char *const sub_exts[] = {"utf", "utf8", "utf-8", "idx", "sub", "srt",
                                        "smi", "rt", "txt", "ssa", "aqt", "jss",
-                                       "js", "ass", NULL};
+                                       "js", "ass", "mks", NULL};
 
 static bool is_sub_ext(bstr ext)
 {
