@@ -71,6 +71,7 @@ extern const vf_info_t vf_info_dlopen;
 extern const vf_info_t vf_info_lavfi;
 extern const vf_info_t vf_info_vaapi;
 extern const vf_info_t vf_info_vapoursynth;
+extern const vf_info_t vf_info_vapoursynth_lazy;
 extern const vf_info_t vf_info_vdpaupp;
 extern const vf_info_t vf_info_buffer;
 
@@ -114,9 +115,12 @@ static const vf_info_t *const filter_list[] = {
 #if HAVE_DLOPEN
     &vf_info_dlopen,
 #endif
-#if HAVE_VAPOURSYNTH
+#if HAVE_VAPOURSYNTH_CORE && HAVE_VAPOURSYNTH
     &vf_info_vapoursynth,
 #endif
+#if HAVE_VAPOURSYNTH_CORE && HAVE_VAPOURSYNTH_LAZY
+    &vf_info_vapoursynth_lazy,
+#endif
 #if HAVE_VAAPI_VPP
     &vf_info_vaapi,
 #endif
@@ -170,10 +174,11 @@ int vf_control_by_label(struct vf_chain *c,int cmd, void *arg, bstr label)
     char *label_str = bstrdup0(NULL, label);
     struct vf_instance *cur = vf_find_by_label(c, label_str);
     talloc_free(label_str);
-    if (cur)
-        return cur->control(cur, cmd, arg);
-    else
+    if (cur) {
+        return cur->control ? cur->control(cur, cmd, arg) : CONTROL_NA;
+    } else {
         return CONTROL_UNKNOWN;
+    }
 }
 
 static void vf_control_all(struct vf_chain *c, int cmd, void *arg)
@@ -226,25 +231,6 @@ static int vf_default_query_format(struct vf_instance *vf, unsigned int fmt)
     return vf_next_query_format(vf, fmt);
 }
 
-static void fmt_cat(char *b, size_t bs, struct mp_image_params *p)
-{
-    if (p && p->imgfmt) {
-        mp_snprintf_cat(b, bs, "%dx%d", p->w, p->h);
-        if (p->w != p->d_w || p->h != p->d_h)
-            mp_snprintf_cat(b, bs, "->%dx%d", p->d_w, p->d_h);
-        mp_snprintf_cat(b, bs, " %s", mp_imgfmt_to_name(p->imgfmt));
-        mp_snprintf_cat(b, bs, " %s/%s", mp_csp_names[p->colorspace],
-                        mp_csp_levels_names[p->colorlevels]);
-        mp_snprintf_cat(b, bs, " CL=%d", (int)p->chroma_location);
-        if (p->outputlevels)
-            mp_snprintf_cat(b, bs, " out=%s", mp_csp_levels_names[p->outputlevels]);
-        if (p->rotate)
-            mp_snprintf_cat(b, bs, " rot=%d", p->rotate);
-    } else {
-        mp_snprintf_cat(b, bs, "???");
-    }
-}
-
 void vf_print_filter_chain(struct vf_chain *c, int msglevel,
                            struct vf_instance *vf)
 {
@@ -253,13 +239,13 @@ void vf_print_filter_chain(struct vf_chain *c, int msglevel,
 
     char b[128] = {0};
 
-    fmt_cat(b, sizeof(b), &c->input_params);
+    mp_snprintf_cat(b, sizeof(b), "%s", mp_image_params_to_str(&c->input_params));
     mp_msg(c->log, msglevel, " [vd] %s\n", b);
 
     for (vf_instance_t *f = c->first; f; f = f->next) {
         b[0] = '\0';
         mp_snprintf_cat(b, sizeof(b), " [%s] ", f->info->name);
-        fmt_cat(b, sizeof(b), &f->fmt_out);
+        mp_snprintf_cat(b, sizeof(b), "%s", mp_image_params_to_str(&f->fmt_out));
         if (f->autoinserted)
             mp_snprintf_cat(b, sizeof(b), " [a]");
         if (f == vf)
@@ -433,14 +419,12 @@ int vf_filter_frame(struct vf_chain *c, struct mp_image *img)
     return vf_do_filter(c->first, img);
 }
 
-// Output the next queued image (if any) from the full filter chain.
-// The frame can be retrieved with vf_read_output_frame().
-//  eof: if set, assume there's no more input i.e. vf_filter_frame() will
-//       not be called (until reset) - flush all internally delayed frames
-//  returns: -1: error, 0: no output, 1: output available
-int vf_output_frame(struct vf_chain *c, bool eof)
+// Similar to vf_output_frame(), but only ensure that the filter "until" has
+// output, instead of the end of the filter chain.
+static int vf_output_frame_until(struct vf_chain *c, struct vf_instance *until,
+                                 bool eof)
 {
-    if (c->output)
+    if (until->num_out_queued)
         return 1;
     if (c->initialized < 1)
         return -1;
@@ -456,27 +440,62 @@ int vf_output_frame(struct vf_chain *c, bool eof)
             }
             if (vf_has_output_frame(cur))
                 last = cur;
+            if (cur == until)
+                break;
         }
         if (!last)
             return 0;
-        struct mp_image *img = vf_dequeue_output_frame(last);
-        if (!last->next) {
-            c->output = img;
-            return !!c->output;
-        }
-        int r = vf_do_filter(last->next, img);
+        if (last == until)
+            return 1;
+        int r = vf_do_filter(last->next, vf_dequeue_output_frame(last));
         if (r < 0)
             return r;
     }
 }
 
+// Output the next queued image (if any) from the full filter chain.
+// The frame can be retrieved with vf_read_output_frame().
+//  eof: if set, assume there's no more input i.e. vf_filter_frame() will
+//       not be called (until reset) - flush all internally delayed frames
+//  returns: -1: error, 0: no output, 1: output available
+int vf_output_frame(struct vf_chain *c, bool eof)
+{
+    return vf_output_frame_until(c, c->last, eof);
+}
+
 struct mp_image *vf_read_output_frame(struct vf_chain *c)
 {
-    if (!c->output)
+    if (!c->last->num_out_queued)
         vf_output_frame(c, false);
-    struct mp_image *res = c->output;
-    c->output = NULL;
-    return res;
+    return vf_dequeue_output_frame(c->last);
+}
+
+// Some filters (vf_vapoursynth) filter on separate threads, and may need new
+// input from the decoder, even though the core does not need a new output image
+// yet (this is required to get proper pipelining in the filter). If the filter
+// needs new data, it will call c->wakeup_callback, which in turn causes the
+// core to recheck the filter chain, calling this function. Each filter is asked
+// whether it needs a frame (with vf->needs_input), and if so, it will try to
+// feed it a new frame. If this fails, it will request a new frame from the
+// core by returning 1.
+// returns -1: error, 0: nothing needed, 1: add new frame with vf_filter_frame()
+int vf_needs_input(struct vf_chain *c)
+{
+    struct vf_instance *prev = c->first;
+    for (struct vf_instance *cur = c->first; cur; cur = cur->next) {
+        while (cur->needs_input && cur->needs_input(cur)) {
+            // Get frames from preceding filters, or if there are none,
+            // request new frames from decoder.
+            int r = vf_output_frame_until(c, prev, false);
+            if (r < 1)
+                return r < 0 ? -1 : 1;
+            r = vf_do_filter(cur, vf_dequeue_output_frame(prev));
+            if (r < 0)
+                return r;
+        }
+        prev = cur;
+    }
+    return 0;
 }
 
 static void vf_forget_frames(struct vf_instance *vf)
@@ -490,7 +509,6 @@ static void vf_chain_forget_frames(struct vf_chain *c)
 {
     for (struct vf_instance *cur = c->first; cur; cur = cur->next)
         vf_forget_frames(cur);
-    mp_image_unrefp(&c->output);
 }
 
 void vf_seek_reset(struct vf_chain *c)
@@ -701,12 +719,13 @@ struct vf_chain *vf_new(struct mpv_global *global)
         .query_format = input_query_format,
     };
     static const struct vf_info out = { .name = "out" };
-    c->first->next = talloc(c, struct vf_instance);
-    *c->first->next = (struct vf_instance) {
+    c->last = talloc(c, struct vf_instance);
+    *c->last = (struct vf_instance) {
         .info = &out,
         .query_format = output_query_format,
         .priv = (void *)c,
     };
+    c->first->next = c->last;
     return c;
 }
 
