@@ -40,6 +40,11 @@ static bool check_hwdec(struct gl_hwdec *hw)
         return false;
     }
 
+    if (hw->mpgl->gl->version < 300) {
+        MP_ERR(hw, "need >= OpenGL 3.0 for core rectangle texture support\n");
+        return false;
+    }
+
     if (!vo->cocoa) {
         MP_ERR(hw, "need cocoa opengl backend to be active");
         return false;
