@@ -47,6 +47,7 @@
 #include "video/vfcap.h"
 #include "sub/osd.h"
 #include "osdep/io.h"
+#include "osdep/threads.h"
 
 extern const struct vo_driver video_out_x11;
 extern const struct vo_driver video_out_vdpau;
@@ -128,6 +129,7 @@ struct vo_internal {
     bool request_redraw;            // redraw request from player to VO
     bool want_redraw;               // redraw request from VO to player
     bool paused;
+    int queued_events;
 
     int64_t flip_queue_offset; // queue flip events at most this much in advance
 
@@ -277,6 +279,8 @@ struct vo *init_best_video_out(struct mpv_global *global,
 autoprobe:
     // now try the rest...
     for (int i = 0; video_out_drivers[i]; i++) {
+        if (video_out_drivers[i] == &video_out_null)
+            break;
         struct vo *vo = vo_create(global, input_ctx, osd, encode_lavc_ctx,
                                   (char *)video_out_drivers[i]->name, NULL);
         if (vo)
@@ -559,12 +563,12 @@ static bool render_frame(struct vo *vo)
     int64_t next_vsync = prev_sync(vo, mp_time_us()) + in->vsync_interval;
     int64_t end_time = pts + duration;
 
-    if (!(vo->global->opts->frame_dropping & 1) || !in->hasframe_rendered ||
-        vo->driver->untimed || vo->driver->encode)
+    if (!in->hasframe_rendered)
         duration = -1; // disable framedrop
 
     in->dropped_frame = duration >= 0 && end_time < next_vsync;
     in->dropped_frame &= !(vo->driver->caps & VO_CAP_FRAMEDROP);
+    in->dropped_frame &= (vo->global->opts->frame_dropping & 1);
     // Even if we're hopelessly behind, rather degrade to 10 FPS playback,
     // instead of just freezing the display forever.
     in->dropped_frame &= mp_time_us() - in->last_flip < 100 * 1000;
@@ -666,6 +670,8 @@ static void *vo_thread(void *ptr)
     struct vo *vo = ptr;
     struct vo_internal *in = vo->in;
 
+    mpthread_set_name("vo");
+
     int r = vo->driver->preinit(vo) ? -1 : 0;
     mp_rendezvous(vo, r); // init barrier
     if (r < 0)
@@ -835,6 +841,30 @@ int64_t vo_get_vsync_interval(struct vo *vo)
     return vo->in->vsync_interval;
 }
 
+// Set specific event flags, and wakeup the playback core if needed.
+// vo_query_and_reset_events() can retrieve the events again.
+void vo_event(struct vo *vo, int event)
+{
+    struct vo_internal *in = vo->in;
+    pthread_mutex_lock(&in->lock);
+    if ((in->queued_events & event & VO_EVENTS_USER) != (event & VO_EVENTS_USER))
+        mp_input_wakeup(vo->input_ctx);
+    in->queued_events |= event;
+    pthread_mutex_unlock(&in->lock);
+}
+
+// Check event flags set with vo_event(). Return the mask of events that was
+// set and included in the events parameter. Clear the returned events.
+int vo_query_and_reset_events(struct vo *vo, int events)
+{
+    struct vo_internal *in = vo->in;
+    pthread_mutex_lock(&in->lock);
+    int r = in->queued_events & events;
+    in->queued_events &= ~(unsigned)r;
+    pthread_mutex_unlock(&in->lock);
+    return r;
+}
+
 /**
  * \brief lookup an integer in a table, table must have 0 as the last key
  * \param key key to search for
