@@ -111,7 +111,7 @@ static void check_pattern(struct vo *vo, int item)
         p->matches++;
     } else {
         p->mismatches++;
-        MP_WARN(vo, "wrong pattern, exptected %d got %d (hit: %d, mis: %d)\n",
+        MP_WARN(vo, "wrong pattern, expected %d got %d (hit: %d, mis: %d)\n",
                 expected, item, p->matches, p->mismatches);
     }
 }
@@ -197,7 +197,7 @@ static bool config_window(struct gl_priv *p, int flags)
     if (p->use_gl_debug)
         flags |= VOFLAG_GL_DEBUG;
 
-    int mpgl_caps = MPGL_CAP_GL21 | MPGL_CAP_TEX_RG;
+    int mpgl_caps = MPGL_CAP_GL21;
     if (!p->allow_sw)
         mpgl_caps |= MPGL_CAP_NO_SW;
     return mpgl_config_window(p->glctx, mpgl_caps, flags);
@@ -225,12 +225,12 @@ static int reconfig(struct vo *vo, struct mp_image_params *params, int flags)
         p->glctx->register_resize_callback(vo, video_resize_redraw_callback);
     }
 
+    resize(p);
+
     gl_video_config(p->renderer, params);
 
     p->vo_flipped = !!(flags & VOFLAG_FLIPPING);
 
-    resize(p);
-
     mpgl_unlock(p->glctx);
 
     return 0;
@@ -439,6 +439,7 @@ static int control(struct vo *vo, uint32_t request, void *data)
         get_and_update_icc_profile(vo, p->icc_opts);
         vo->want_redraw = true;
     }
+    vo_event(vo, events);
     mpgl_unlock(p->glctx);
 
     return r;
