@@ -1687,7 +1687,7 @@ static int initGl(struct vo *vo, uint32_t d_width, uint32_t d_height)
     }
 
     if (gl->BindTexture) {
-        p->osd = mpgl_osd_init(gl, vo->log, vo->osd);
+        p->osd = mpgl_osd_init(gl, vo->log, vo->osd, true);
         p->osd->scaled = p->scaled_osd;
     }
 
@@ -2161,6 +2161,7 @@ static int control(struct vo *vo, uint32_t request, void *data)
         resize(vo, vo->dwidth, vo->dheight);
     if (events & VO_EVENT_EXPOSE)
         vo->want_redraw = true;
+    vo_event(vo, events);
 
     return r;
 }
