@@ -46,10 +46,6 @@
 #include "input/event.h"
 #include "input/keycodes.h"
 
-#define MOD_SHIFT_MASK      0x01
-#define MOD_ALT_MASK        0x02
-#define MOD_CONTROL_MASK    0x04
-
 static int lookupkey(int key);
 
 static void hide_cursor(struct vo_wayland_state * wl);
@@ -282,24 +278,34 @@ static void keyboard_handle_key(void *data,
                                 uint32_t state)
 {
     struct vo_wayland_state *wl = data;
-    uint32_t code, num_syms;
-    int mpkey;
 
-    const xkb_keysym_t *syms;
-    xkb_keysym_t sym;
+    uint32_t code = code = key + 8;
+    xkb_keysym_t sym = xkb_state_key_get_one_sym(wl->input.xkb.state, code);
 
-    code = key + 8;
-    num_syms = xkb_state_key_get_syms(wl->input.xkb.state, code, &syms);
+    int mpmod = state == WL_KEYBOARD_KEY_STATE_PRESSED ? MP_KEY_STATE_DOWN
+                                                       : MP_KEY_STATE_UP;
 
-    sym = XKB_KEY_NoSymbol;
-    if (num_syms == 1)
-        sym = syms[0];
+    static const char *mod_names[] = {XKB_MOD_NAME_SHIFT, XKB_MOD_NAME_CTRL,
+                                      XKB_MOD_NAME_ALT, XKB_MOD_NAME_LOGO, 0};
+    static int mods[] = {MP_KEY_MODIFIER_SHIFT, MP_KEY_MODIFIER_CTRL,
+                         MP_KEY_MODIFIER_ALT, MP_KEY_MODIFIER_META, 0};
 
-    if (sym != XKB_KEY_NoSymbol && (mpkey = lookupkey(sym))) {
-        if (state == WL_KEYBOARD_KEY_STATE_PRESSED)
-            mp_input_put_key(wl->vo->input_ctx, mpkey | MP_KEY_STATE_DOWN);
-        else
-            mp_input_put_key(wl->vo->input_ctx, mpkey | MP_KEY_STATE_UP);
+    for (int n = 0; mods[n]; n++) {
+        xkb_mod_index_t index =
+            xkb_keymap_mod_get_index(wl->input.xkb.keymap, mod_names[n]);
+        if (!xkb_state_mod_index_is_consumed(wl->input.xkb.state, code, index)
+            && xkb_state_mod_index_is_active(wl->input.xkb.state, index,
+                                             XKB_STATE_MODS_DEPRESSED))
+            mpmod |= mods[n];
+    }
+
+    int mpkey = lookupkey(sym);
+    if (mpkey) {
+        mp_input_put_key(wl->vo->input_ctx, mpkey | mpmod);
+    } else {
+        char s[80];
+        if (xkb_keysym_to_utf8(sym, s, sizeof(s)) > 0)
+            mp_input_put_key_utf8(wl->vo->input_ctx, mpmod, bstr0(s));
     }
 }
 
@@ -717,6 +723,9 @@ static void schedule_resize(struct vo_wayland_state *wl,
     int32_t minimum_size = 150;
     int32_t x, y;
     float temp_aspect = width / (float) MPMAX(height, 1);
+    float win_aspect = wl->window.aspect;
+    if (win_aspect <= 0)
+        win_aspect = 1;
 
     MP_DBG(wl, "schedule resize: %dx%d\n", width, height);
 
@@ -734,7 +743,7 @@ static void schedule_resize(struct vo_wayland_state *wl,
     switch (edges) {
         case WL_SHELL_SURFACE_RESIZE_TOP:
         case WL_SHELL_SURFACE_RESIZE_BOTTOM:
-            width = wl->window.aspect * height;
+            width = win_aspect * height;
             break;
         case WL_SHELL_SURFACE_RESIZE_LEFT:
         case WL_SHELL_SURFACE_RESIZE_RIGHT:
@@ -742,13 +751,13 @@ static void schedule_resize(struct vo_wayland_state *wl,
         case WL_SHELL_SURFACE_RESIZE_TOP_RIGHT:
         case WL_SHELL_SURFACE_RESIZE_BOTTOM_LEFT:
         case WL_SHELL_SURFACE_RESIZE_BOTTOM_RIGHT:
-            height = (1 / wl->window.aspect) * width;
+            height = (1 / win_aspect) * width;
             break;
         default:
             if (wl->window.aspect < temp_aspect)
                 width = wl->window.aspect * height;
             else
-                height = (1 / wl->window.aspect) * width;
+                height = (1 / win_aspect) * width;
             break;
     }
 
