@@ -127,6 +127,7 @@ static void vo_x11_selectinput_witherr(struct vo *vo, Display *display,
 static void vo_x11_setlayer(struct vo *vo, bool ontop);
 
 #define XA(x11, s) (XInternAtom((x11)->display, # s, False))
+#define XAs(x11, s) XInternAtom((x11)->display, s, False)
 
 #define RC_W(rc) ((rc).x1 - (rc).x0)
 #define RC_H(rc) ((rc).y1 - (rc).y0)
@@ -227,7 +228,7 @@ static void vo_set_cursor_hidden(struct vo *vo, bool cursor_hidden)
 
     x11->mouse_cursor_hidden = cursor_hidden;
 
-    if (vo->opts->WinID == 0 || !win)
+    if (x11->parent == x11->rootwin || !win)
         return;                 // do not hide if playing on the root window
 
     if (x11->mouse_cursor_hidden) {
@@ -288,7 +289,7 @@ static int vo_wm_detect(struct vo *vo)
     Atom *args = NULL;
     Window win = x11->rootwin;
 
-    if (vo->opts->WinID >= 0)
+    if (x11->parent)
         return 0;
 
 // -- supports layers
@@ -308,7 +309,7 @@ static int vo_wm_detect(struct vo *vo)
                             &nitems);
     if (args) {
         MP_VERBOSE(x11, "Detected wm supports NetWM.\n");
-        if (vo->opts->x11_netwm) {
+        if (vo->opts->x11_netwm >= 0) {
             for (i = 0; i < nitems; i++)
                 wm |= net_wm_support_state_test(vo->x11, args[i]);
         } else {
@@ -319,12 +320,19 @@ static int vo_wm_detect(struct vo *vo)
 
     if (wm == 0)
         MP_VERBOSE(x11, "Unknown wm type...\n");
+    if (vo->opts->x11_netwm > 0 && !(wm & vo_wm_FULLSCREEN)) {
+        MP_WARN(x11, "Forcing NetWM FULLSCREEN support.\n");
+        wm |= vo_wm_FULLSCREEN;
+    }
     return wm;
 }
 
 static void xrandr_read(struct vo_x11_state *x11)
 {
 #if HAVE_XRANDR
+    for(int i = 0; i < x11->num_displays; i++)
+        talloc_free(x11->displays[i].name);
+
     x11->num_displays = 0;
 
     if (x11->xrandr_event < 0) {
@@ -361,14 +369,20 @@ static void xrandr_read(struct vo_x11_state *x11)
                     continue;
                 if (x11->num_displays >= MAX_DISPLAYS)
                     continue;
+                double vTotal = m.vTotal;
+                if (m.modeFlags & RR_DoubleScan)
+                    vTotal *= 2;
+                if (m.modeFlags & RR_Interlace)
+                    vTotal /= 2;
                 struct xrandr_display d = {
                     .rc = { crtc->x, crtc->y,
                             crtc->x + crtc->width, crtc->y + crtc->height },
-                    .fps = m.dotClock / (m.hTotal * (double)m.vTotal),
+                    .fps = m.dotClock / (m.hTotal * vTotal),
+                    .name = talloc_strdup(x11, out->name),
                 };
                 int num = x11->num_displays++;
-                MP_VERBOSE(x11, "Display %d: [%d, %d, %d, %d] @ %f FPS\n",
-                           num, d.rc.x0, d.rc.y0, d.rc.x1, d.rc.y1, d.fps);
+                MP_VERBOSE(x11, "Display %d (%s): [%d, %d, %d, %d] @ %f FPS\n",
+                           num, d.name, d.rc.x0, d.rc.y0, d.rc.x1, d.rc.y1, d.fps);
                 x11->displays[num] = d;
             }
         }
@@ -489,6 +503,9 @@ int vo_x11_init(struct vo *vo)
     x11->screen = DefaultScreen(x11->display);  // screen ID
     x11->rootwin = RootWindow(x11->display, x11->screen);   // root window ID
 
+    if (vo->opts->WinID >= 0)
+        x11->parent = vo->opts->WinID ? vo->opts->WinID : x11->rootwin;
+
     if (!opts->native_keyrepeat) {
         Bool ok = False;
         XkbSetDetectableAutoRepeat(x11->display, True, &ok);
@@ -594,7 +611,7 @@ static void vo_x11_decoration(struct vo *vo, bool d)
 {
     struct vo_x11_state *x11 = vo->x11;
 
-    if (vo->opts->WinID >= 0 || !x11->window)
+    if (x11->parent || !x11->window)
         return;
 
     Atom motif_hints = XA(x11, _MOTIF_WM_HINTS);
@@ -652,11 +669,6 @@ void vo_x11_uninit(struct vo *vo)
 
         XSelectInput(x11->display, x11->window, StructureNotifyMask);
         XDestroyWindow(x11->display, x11->window);
-        XEvent xev;
-        do {
-            XNextEvent(x11->display, &xev);
-        } while (xev.type != DestroyNotify ||
-                 xev.xdestroywindow.event != x11->window);
     }
     if (x11->xic)
         XDestroyIC(x11->xic);
@@ -674,6 +686,8 @@ void vo_x11_uninit(struct vo *vo)
     vo->x11 = NULL;
 }
 
+#define DND_PROPERTY "mpv_dnd_selection"
+
 static void vo_x11_dnd_init_window(struct vo *vo)
 {
     struct vo_x11_state *x11 = vo->x11;
@@ -681,8 +695,6 @@ static void vo_x11_dnd_init_window(struct vo *vo)
     Atom version = DND_VERSION;
     XChangeProperty(x11->display, x11->window, XA(x11, XdndAware), XA_ATOM,
                     32, PropModeReplace, (unsigned char *)&version, 1);
-
-    x11->dnd_property = XInternAtom(x11->display, "mpv_dnd_selection", False);
 }
 
 static void dnd_select_format(struct vo_x11_state *x11, Atom *args, int items)
@@ -747,7 +759,7 @@ static void vo_x11_dnd_handle_message(struct vo *vo, XClientMessageEvent *ce)
     } else if (ce->message_type == XA(x11, XdndDrop)) {
         x11->dnd_src_window = ce->data.l[0];
         XConvertSelection(x11->display, XA(x11, XdndSelection),
-                          x11->dnd_requested_format, x11->dnd_property,
+                          x11->dnd_requested_format, XAs(x11, DND_PROPERTY),
                           x11->window, ce->data.l[2]);
     } else if (ce->message_type == XA(x11, XdndLeave)) {
         dnd_reset(vo);
@@ -764,11 +776,11 @@ static void vo_x11_dnd_handle_selection(struct vo *vo, XSelectionEvent *se)
     bool success = false;
 
     if (se->selection == XA(x11, XdndSelection) &&
-        se->property == x11->dnd_property &&
+        se->property == XAs(x11, DND_PROPERTY) &&
         se->target == x11->dnd_requested_format)
     {
         int nitems;
-        void *prop = x11_get_property(x11, x11->window, x11->dnd_property,
+        void *prop = x11_get_property(x11, x11->window, XAs(x11, DND_PROPERTY),
                                       x11->dnd_requested_format, 8, &nitems);
         if (prop) {
             // No idea if this is guaranteed to be \0-padded, so use bstr.
@@ -840,7 +852,7 @@ int vo_x11_check_events(struct vo *vo)
             if (x11->window == None)
                 break;
             vo_x11_update_geometry(vo);
-            if (Event.xconfigure.window == (Window)vo->opts->WinID) {
+            if (Event.xconfigure.window == x11->parent) {
                 XMoveResizeWindow(x11->display, x11->window,
                                   x11->winrc.x0, x11->winrc.y0,
                                   RC_W(x11->winrc), RC_H(x11->winrc));
@@ -943,11 +955,13 @@ int vo_x11_check_events(struct vo *vo)
             vo_x11_dnd_handle_selection(vo, &Event.xselection);
             break;
         case PropertyNotify:
-            if (Event.xproperty.atom == x11->atom_frame_exts) {
-                if (!x11->pseudo_mapped && vo->opts->WinID < 0) {
+            if (Event.xproperty.atom == XA(x11, _NET_FRAME_EXTENTS)) {
+                if (!x11->pseudo_mapped && !x11->parent) {
                     MP_VERBOSE(x11, "not waiting for MapNotify\n");
                     x11->pseudo_mapped = true;
                 }
+            } else if (Event.xproperty.atom == XA(x11, _NET_WM_STATE)) {
+                x11->pending_vo_events |= VO_EVENT_WIN_STATE;
             }
             break;
         default:
@@ -974,13 +988,13 @@ static void vo_x11_sizehint(struct vo *vo, struct mp_rect rc, bool override_pos)
     struct mp_vo_opts *opts = vo->opts;
     struct vo_x11_state *x11 = vo->x11;
 
-    if (!x11->window)
+    if (!x11->window || x11->parent)
         return;
 
     bool force_pos = opts->geometry.xy_valid ||     // explicitly forced by user
                      opts->force_window_position || // resize -> reset position
                      opts->screen_id >= 0 ||        // force onto screen area
-                     opts->WinID >= 0 ||            // force to fill parent
+                     x11->parent ||                 // force to fill parent
                      override_pos;                  // for fullscreen and such
 
     XSizeHints *hint = XAllocSizeHints();
@@ -995,7 +1009,7 @@ static void vo_x11_sizehint(struct vo *vo, struct mp_rect rc, bool override_pos)
     hint->max_width = 0;
     hint->max_height = 0;
 
-    if (opts->keepaspect) {
+    if (opts->keepaspect && opts->keepaspect_window) {
         hint->flags |= PAspect;
         hint->min_aspect.x = hint->width;
         hint->min_aspect.y = hint->height;
@@ -1195,16 +1209,15 @@ static void vo_x11_create_window(struct vo *vo, XVisualInfo *vis,
         .colormap = x11->colormap,
     };
 
-    Window parent = vo->opts->WinID >= 0 ? vo->opts->WinID : x11->rootwin;
+    Window parent = x11->parent;
+    if (!parent)
+        parent = x11->rootwin;
 
     x11->window =
         XCreateWindow(x11->display, parent, rc.x0, rc.y0, RC_W(rc), RC_H(rc), 0,
                       vis->depth, CopyFromParent, vis->visual, xswamask, &xswa);
     Atom protos[1] = {XA(x11, WM_DELETE_WINDOW)};
     XSetWMProtocols(x11->display, x11->window, protos, 1);
-    x11->f_gc = XCreateGC(x11->display, x11->window, 0, 0);
-    x11->vo_gc = XCreateGC(x11->display, x11->window, 0, NULL);
-    XSetForeground(x11->display, x11->f_gc, 0);
 
     if (x11->mouse_cursor_hidden) {
         x11->mouse_cursor_hidden = false;
@@ -1218,11 +1231,11 @@ static void vo_x11_create_window(struct vo *vo, XVisualInfo *vis,
                              NULL);
     }
 
-    vo_x11_set_wm_icon(x11);
-    vo_x11_update_window_title(vo);
-    vo_x11_dnd_init_window(vo);
-
-    x11->atom_frame_exts = XA(x11, _NET_FRAME_EXTENTS);
+    if (!x11->parent) {
+        vo_x11_set_wm_icon(x11);
+        vo_x11_update_window_title(vo);
+        vo_x11_dnd_init_window(vo);
+    }
 }
 
 static void vo_x11_map_window(struct vo *vo, struct mp_rect rc)
@@ -1259,7 +1272,7 @@ static void vo_x11_map_window(struct vo *vo, struct mp_rect rc)
                  LeaveWindowMask;
     if (mp_input_mouse_enabled(vo->input_ctx))
         events |= PointerMotionMask | ButtonPressMask | ButtonReleaseMask;
-    if (mp_input_x11_keyboard_enabled(vo->input_ctx))
+    if (mp_input_vo_keyboard_enabled(vo->input_ctx))
         events |= KeyPressMask | KeyReleaseMask;
     vo_x11_selectinput_witherr(vo, x11->display, x11->window, events);
     XMapWindow(x11->display, x11->window);
@@ -1285,9 +1298,6 @@ static void vo_x11_highlevel_resize(struct vo *vo, struct mp_rect rc)
     } else {
         vo_x11_move_resize(vo, reset_pos, true, rc);
     }
-
-    vo_x11_update_geometry(vo);
-    update_vo_size(vo);
 }
 
 static void wait_until_mapped(struct vo *vo)
@@ -1325,13 +1335,13 @@ void vo_x11_config_vo_window(struct vo *vo, XVisualInfo *vis, int flags,
 
     struct mp_rect rc = geo.win;
 
-    if (opts->WinID >= 0) {
-        if (opts->WinID == 0) {
+    if (x11->parent) {
+        if (x11->parent == x11->rootwin) {
             x11->window = x11->rootwin;
             x11->pseudo_mapped = true;
             XSelectInput(x11->display, x11->window, StructureNotifyMask);
         } else {
-            XSelectInput(x11->display, opts->WinID, StructureNotifyMask);
+            XSelectInput(x11->display, x11->parent, StructureNotifyMask);
         }
         vo_x11_update_geometry(vo);
         rc = x11->winrc;
@@ -1343,6 +1353,12 @@ void vo_x11_config_vo_window(struct vo *vo, XVisualInfo *vis, int flags,
         x11->winrc = geo.win;
     }
 
+    if (!x11->f_gc && !x11->vo_gc) {
+        x11->f_gc = XCreateGC(x11->display, x11->window, 0, 0);
+        x11->vo_gc = XCreateGC(x11->display, x11->window, 0, NULL);
+        XSetForeground(x11->display, x11->f_gc, 0);
+    }
+
     if (flags & VOFLAG_HIDDEN)
         return;
 
@@ -1410,7 +1426,7 @@ void vo_x11_clearwindow(struct vo *vo, Window vo_window)
 static void vo_x11_setlayer(struct vo *vo, bool ontop)
 {
     struct vo_x11_state *x11 = vo->x11;
-    if (vo->opts->WinID >= 0 || !x11->window)
+    if (x11->parent || !x11->window)
         return;
 
     if (x11->wm_type & (vo_wm_STAYS_ON_TOP | vo_wm_ABOVE)) {
@@ -1454,14 +1470,14 @@ static void vo_x11_update_geometry(struct vo *vo)
     unsigned w, h, dummy_uint;
     int dummy_int;
     Window dummy_win;
-    Window win = vo->opts->WinID > 0 ? vo->opts->WinID : x11->window;
+    Window win = x11->parent ? x11->parent : x11->window;
     if (!win)
         return;
     XGetGeometry(x11->display, win, &dummy_win, &dummy_int, &dummy_int,
                  &w, &h, &dummy_int, &dummy_uint);
     if (w > INT_MAX || h > INT_MAX)
         w = h = 0;
-    if (vo->opts->WinID < 0) {
+    if (!x11->parent) {
         XTranslateCoordinates(x11->display, x11->window, x11->rootwin, 0, 0,
                               &x, &y, &dummy_win);
     }
@@ -1476,6 +1492,8 @@ static void vo_x11_update_geometry(struct vo *vo)
     if (fps != x11->current_display_fps)
         MP_VERBOSE(x11, "Current display FPS: %f\n", fps);
     x11->current_display_fps = fps;
+    // might have changed displays
+    x11->pending_vo_events |= VO_EVENT_WIN_STATE;
 }
 
 static void vo_x11_fullscreen(struct vo *vo)
@@ -1486,7 +1504,7 @@ static void vo_x11_fullscreen(struct vo *vo)
     if (opts->fullscreen == x11->fs)
         return;
     x11->fs = opts->fullscreen; // x11->fs now contains the new state
-    if (opts->WinID >= 0 || !x11->window)
+    if (x11->parent || !x11->window)
         return;
 
     // Save old state before entering fullscreen
@@ -1575,6 +1593,40 @@ int vo_x11_control(struct vo *vo, int *events, int request, void *arg)
         rc.x1 = rc.x0 + s[0];
         rc.y1 = rc.y0 + s[1];
         vo_x11_highlevel_resize(vo, rc);
+        if (!x11->fs) { // guess new window size, instead of waiting for X
+            x11->winrc.x1 = x11->winrc.x0 + s[0];
+            x11->winrc.y1 = x11->winrc.y0 + s[1];
+        }
+        return VO_TRUE;
+    }
+    case VOCTRL_GET_WIN_STATE: {
+        if (!x11->pseudo_mapped)
+            return VO_FALSE;
+        int num_elems;
+        long *elems = x11_get_property(x11, x11->window, XA(x11, _NET_WM_STATE),
+                                       XA_ATOM, 32, &num_elems);
+        if (elems) {
+            Atom hidden = XA(x11, _NET_WM_STATE_HIDDEN);
+            for (int n = 0; n < num_elems; n++) {
+                if (elems[n] == hidden)
+                    *(int *)arg |= VO_WIN_STATE_MINIMIZED;
+            }
+            XFree(elems);
+        }
+        return VO_TRUE;
+    }
+    case VOCTRL_GET_DISPLAY_NAMES: {
+        if (!x11->pseudo_mapped)
+            return VO_FALSE;
+        char **names = NULL;
+        int displays_spanned = 0;
+        for (int n = 0; n < x11->num_displays; n++) {
+            if (rc_overlaps(x11->displays[n].rc, x11->winrc))
+                MP_TARRAY_APPEND(NULL, names, displays_spanned,
+                                 talloc_strdup(NULL, x11->displays[n].name));
+        }
+        MP_TARRAY_APPEND(NULL, names, displays_spanned, NULL);
+        *(char ***)arg = names;
         return VO_TRUE;
     }
     case VOCTRL_SET_CURSOR_VISIBILITY:
@@ -1587,7 +1639,8 @@ int vo_x11_control(struct vo *vo, int *events, int request, void *arg)
         set_screensaver(x11, true);
         return VO_TRUE;
     case VOCTRL_UPDATE_WINDOW_TITLE:
-        vo_x11_update_window_title(vo);
+        if (!x11->parent)
+            vo_x11_update_window_title(vo);
         return VO_TRUE;
     case VOCTRL_GET_DISPLAY_FPS: {
         double fps = x11->current_display_fps;
