@@ -34,6 +34,7 @@ struct mp_log;
 struct xrandr_display {
     struct mp_rect rc;
     double fps;
+    char *name;
 };
 
 struct vo_x11_state {
@@ -41,6 +42,7 @@ struct vo_x11_state {
     Display *display;
     Window window;
     Window rootwin;
+    Window parent;  // embedded in this foreign window
     int screen;
     int display_is_local;
     int ws_width;
@@ -103,12 +105,9 @@ struct vo_x11_state {
     int ShmCompletionWaitCount;
 
     /* drag and drop */
-    Atom dnd_property;
     Atom dnd_requested_format;
     Window dnd_src_window;
 
-    Atom atom_frame_exts;
-
     /* dragging the window */
     bool win_drag_button1_down;
 };
